﻿using PDFPrintingSample.UWP;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Globalization;
using Windows.Graphics.Display;
using Windows.Graphics.Printing;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Printing;
using System.Windows;
using Windows.UI.Popups;
using Windows.UI.Core;
using Windows.ApplicationModel.Core;

[assembly: Xamarin.Forms.Dependency(typeof(PrintService))]

namespace PDFPrintingSample.UWP
{
    class PrintService : IPrintService
    {
        internal int m_pageCount;
        internal global::Windows.Data.Pdf.PdfDocument _pdfDocument;
        private PrintDocument printDocument;
        private IPrintDocumentSource printDocumentSource;
        string file_Name;
        internal Dictionary<int, UIElement> printPreviewPages = new Dictionary<int, UIElement>();
        private List<string> m_imagePaths = new List<string>();
        private DispatcherTimer m_dispatcherTimer = new DispatcherTimer();
        IRandomAccessStream randomStream;
        public async void Print(Stream inputStream, string fileName)
        {
            inputStream.Position = 0;
            MemoryStream ms = new MemoryStream();
            inputStream.CopyTo(ms);
            ms.Position = 0;
            randomStream = await ConvertToRandomAccessStream(ms);
            IAsyncOperation<global::Windows.Data.Pdf.PdfDocument> result = null;
            result = global::Windows.Data.Pdf.PdfDocument.LoadFromStreamAsync(randomStream);

            result.AsTask().Wait();
            _pdfDocument = result.GetResults();
            result = null;
            m_pageCount = (int)_pdfDocument.PageCount;
            file_Name = fileName;
            try
            {
                UIDispatcher.Execute(async () =>
                {
                    RegisterForPrint();
                    await PrintManager.ShowPrintUIAsync();
                });
            }
            catch
            {
                UIDispatcher.Execute(async () =>
                {
                    RegisterForPrint();
                    PrintManager.ShowPrintUIAsync();
                });
            }
        }
        public async Task<IRandomAccessStream> ConvertToRandomAccessStream(MemoryStream memoryStream)
        {
            var randomAccessStream = new InMemoryRandomAccessStream();
            MemoryStream contentStream = new MemoryStream();
            memoryStream.CopyTo(contentStream);
            using (var outputStream = randomAccessStream.GetOutputStreamAt(0))
            {
                using (var dw = new DataWriter(outputStream))
                {
                    var task = new Task(() => dw.WriteBytes(contentStream.ToArray()));
                    task.Start();

                    await task;
                    await dw.StoreAsync();

                    await outputStream.FlushAsync();
                    await dw.FlushAsync();
                    outputStream.Dispose();
                    dw.DetachStream();
                    dw.Dispose();
                }
            }
            return randomAccessStream;
        }
        PrintManager printMan;
        private void RegisterForPrint()
        {
            printDocument = new PrintDocument();

            // Save the DocumentSource.
            printDocumentSource = printDocument.DocumentSource;
         
            // Add an event handler which provides all final print pages.
            printDocument.AddPages += AddPrintPages;

            // Create a PrintManager and add a handler for printing initialization.
            printMan = PrintManager.GetForCurrentView();
            printMan.PrintTaskRequested += PrintTaskRequested;
        }
        
        Image imageCtrl = new Image();
        private async void AddPrintPages(object sender, AddPagesEventArgs e)
        {
            try
            {
                await PrepareForPrint(0, m_pageCount);
                PrintDocument printDoc = (PrintDocument)sender;
                printDoc.AddPagesComplete();
            }
            catch
            {
                PrintDocument printDoc = (PrintDocument)sender;
                printDoc.InvalidatePreview();
            }
        }
        private async Task<int> PrepareForPrint(int startIndex, int count)
        {
            StorageFolder tempFolder = ApplicationData.Current.TemporaryFolder;
            int result = await PrepareForPrint(startIndex, count, tempFolder);
            tempFolder = null;
            return result;
        }
        private async Task<int> PrepareForPrint(int p, int count, StorageFolder tempfolder)
        {
            for (int i = p; i < count; i++)
            {
                float zoomfactor = 1;
                double customZoomFactor = zoomfactor * 96 / DisplayProperties.LogicalDpi;                
                ApplicationLanguages.PrimaryLanguageOverride = CultureInfo.InvariantCulture.TwoLetterISOLanguageName;
                var pdfPage = _pdfDocument.GetPage(uint.Parse(i.ToString()));
                double pdfPagePreferredZoom = pdfPage.PreferredZoom /** m_pdfViewer.PrinterSettings.QualityFactor*/;
                IRandomAccessStream randomStream = new InMemoryRandomAccessStream();
                global::Windows.Data.Pdf.PdfPageRenderOptions pdfPageRenderOptions = new global::Windows.Data.Pdf.PdfPageRenderOptions();
                Size pdfPageSize = pdfPage.Size;
                pdfPageRenderOptions.DestinationHeight = (uint)(pdfPageSize.Height * pdfPagePreferredZoom);
                pdfPageRenderOptions.DestinationWidth = (uint)(pdfPageSize.Width * pdfPagePreferredZoom);
                await pdfPage.RenderToStreamAsync(randomStream, pdfPageRenderOptions);
                imageCtrl = new Image();
                BitmapImage src = new BitmapImage();
                randomStream.Seek(0);
                src.SetSource(randomStream);
                imageCtrl.Source = src;
                var DisplayInformation = Windows.Graphics.Display.DisplayInformation.GetForCurrentView();

                var dpi = DisplayInformation.LogicalDpi/96;
                imageCtrl.Height = src.PixelHeight/dpi;
                imageCtrl.Width = src.PixelWidth/dpi;
                randomStream.Dispose();
                pdfPage.Dispose();
                printDocument.AddPage(imageCtrl);
            }
            return 0;
        }
        PrintTask printTask;
        MessageDialog msgDialog;
        private async void PrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs e)
        {

            printTask = e.Request.CreatePrintTask(file_Name, sourceRequested => sourceRequested.SetSource(printDocumentSource));

                printTask.Completed += printTask_Completed;
            
        }

     

        private void printTask_Completed(PrintTask sender, PrintTaskCompletedEventArgs args)
        {
            printTask.Completed -= printTask_Completed;
            printMan.PrintTaskRequested -= PrintTaskRequested;


            UIDispatcher.Execute(async () =>
            {

                
                msgDialog = new MessageDialog("Printing operation has been completed");
                await msgDialog.ShowAsync();                

            });
            // PrintManager printMan = PrintManager.GetForCurrentView();
            //printMan.PrintTaskRequested -= PrintTaskRequested;
        }  
    }
    internal class UIDispatcher
    {
        private static CoreDispatcher Dispatcher = CoreApplication.MainView.CoreWindow.Dispatcher;
        /// <summary>
        /// Executes the action using UIElement.
        /// </summary>
        /// <param name="action">An Action.</param>
        internal static void Execute(Action action)
        {
            if (CoreApplication.MainView.CoreWindow == null
                || Dispatcher.HasThreadAccess)
                action();
            else
                Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => action()).AsTask().Wait();
        }
    }
}
